//
//  OMAutomatonState.h
//
//  Created by Samuel Grau on 1/23/12.
//  Copyright (c) 2012 Samuel GRAU. All rights reserved.
//

#import <Foundation/Foundation.h>

@class OMATransition;

/*!
 * Represents a state in an automata. Each state has a kind, along with zero
 * or more children and an (optional) supplementary data item.
 * 
 * @author Samuel GRAU
 * 
 */
@interface OMAState : NSObject {
    NSMutableSet * _transitions;
    BOOL _isAcceptState;
}

@property (nonatomic, assign) BOOL isAcceptState;

/** 
 * Resets transition set. 
 */
- (void)resetTransitions;

/** 
 * Returns the set of outgoing transitions. 
 * Subsequent changes are reflected in the automaton.
 * @return transition set
 */
- (NSMutableSet *)transitions;

/**
 * Adds an outgoing transition.
 * @param t transition
 */
- (void)addTransition:(OMATransition *)transition;

/** 
 * Performs lookup in transitions, assuming determinism. 
 * @param value value object to look up
 * @return destination state, nil if no matching outgoing transition
 */
- (OMAState *)step:(id)value;

/**
 * Adds an epsilon transition between self and the `toState`
 * @param toState The destination state of the epsilon transition
 */
- (void)addEpsilon:(OMAState *)toState;

@end
